package PhotoPub::Handlers;
require Exporter;
@ISA = qw(Exporter);
@EXPORT_OK = qw(&installhandler &handlephoto &listhandlephoto &forallhandlers);

use PhotoPub::Global qw(&register_init $error);

my %handlers;
# cache to indicate whether a path has had dynamicloader run on it
my %dynamicloaded; 

#   **Warning**
# Our debug level must be less than or equal to that of our calling 
# script, or problems (eg output before headers etc) will occur
my $debug = 0;

# should rename handlephoto/listhandlephoto to handlename/listhandlename
my @handlertypes=('dynamicloader'
		 ,'imagefile'         # handlephoto
		 ,'photolist'         # listhandlephoto
		 ,'enumsubpaths'      # forallhandlers
		 ,'cachefileprefix'   # handlephoto
		 ,'exifcachefile'     # handlephoto
		 ,'photogroups'       # forallhandlers
		 ,'forcemogrify'      # handlephoto
		 ,'rangetemplate'     # handlephoto
		 ,'albumtemplate'     # handlephoto
#		 ,'annotations'       # handlephoto
                 ,'description'       # handlephoto
                 ,'options'           # handlephoto
		 ,'album'             # listhandlephoto
		 ,'rangetitle'        # handlephoto
                 ,'whentaken'         # handlephoto
		 ,'canonicalname'     # handlephoto
                 ,'test'              # handlephoto
                 );

&register_init('startup',sub {
		 foreach my $type (@handlertypes) {
		   $handlers{$type}={};
		 }
                 %dynamicloaded=();
	       });

# $direction=1 => upwards; $direction=-1 => downwards
sub walkhandlers_internal {
  my ($direction,$type,$components,$sub)=@_;

  my $limit=scalar(@$components);
  my $i;
  my ($finished,$loop);
  
  if($direction==-1) {
    $i=$limit;
    $finished=sub { return $i<0; };
    $loop=sub { $i--; };
  } else {
    $i=0;
    $finished=sub { return $i>$limit; };
    $loop=sub { $i++; };
  }

  if( $debug >= 3 ) {
    print " internal: \$type=$type \$components=$components \$i=$i \$sub";
    print "=$sub\n";
  }

  while(!&$finished()) {
    my $path=&makepathstring(@$components[0..($i-1)]);
    if($type eq 'dynamicloader') {
       if(exists $dynamicloaded{$path}) {
	  &$loop();
	  next;
       } else {
	  $dynamicloaded{$path}=1;
       }
    }
    if( $debug >= 2 ) {
      print "  path is $path\n";
    }

    if(exists $handlers{$type}->{$path}) {
      foreach my $handler (@{$handlers{$type}->{$path}}) {
	my $val=$handler->(join('/',@$components[$i..$#$components]));
	if(defined $val) {
	  if($debug >= 3) {
	    print " found\n";
	  }
	  my $ret=$sub->($val);
	  if($ret) {
	    if($debug >= 3) {
	      print " terminated\n";
	    }
	    return;
	  }
	}
      }
    }
    &$loop();
  }
}

sub walkhandlers {
  my ($type,$photo,$sub)=@_;

  if( $debug >= 2 ) {
    print "Walking handlers: \$type=$type \$photo=$photo \$sub=$sub\n";
  }

  my $handlers=$handlers{$type}
    or &$error(type => 'internal',
	       message => "Unknown handler type $type");
  my @components=split('/',$photo);
  shift @components; # Remove effect of leading /

  &walkhandlers_internal(1,'dynamicloader',\@components,sub {
			   return 0;
			 });
  &walkhandlers_internal(-1,$type,\@components,$sub);
}

sub installhandler {
  my ($type,$path,$sub)=@_;
  defined $handlers{$type} 
    or &$error("Attempt to register an unknown handler type $type");
  $handlers{$type}->{$path}=[] unless exists $handlers{$type}->{$path};
  unshift(@{$handlers{$type}->{$path}},$sub);
  if( $debug >= 2) {
    print "Installed handler: \$type=$type \$path=$path\n";
  }
}

sub makepathstring {
  my $ret='';
  foreach my $component (@_) {
    $ret.='/'.$component;
  }
  return $ret;
}

# Assumes a type whose handler takes one parameter, which is a photo 
# subpath, and return a scalar. Tries each relevant handler in turn, 
# from most specific to least specific, passing the residual path to each, 
# until one not returning undef is found. Otherwise, returns undef.
sub handlephoto {
  my $ret;
  &walkhandlers(@_,sub {
		  ($ret)=@_;
		  return 1;
		});
  return $ret;
}

# Assumes a type whose handler takes a photo subpath and returns an array
# reference. Iterates over all handlers for the given photo path, most 
# specific to least specific, and sticks all the arrays together, returning
# an array reference too.

sub listhandlephoto {
  my $ret=[];
  &walkhandlers(@_,sub {
		  my ($a)=@_;
		  push @$ret,@$a;
		  return 0;
		});
  return $ret;
}

# Assumes a type whose handler takes no parameters.
# Iterates over all handlers for that type, and for each calls the provided 
# subroutine with the path corresponding to the handler and the result of 
# calling the handler for that path.

# The pathrestrict parameter is used to restrict the paths whose
# corresponding handler is called. Incompatible paths are not called
# at all; thus if pathrestrict is '/ganesh' then the handler for '/stephen'
# won't be called. However, *both* less specific and more specific paths are
# called, thus the root handler and the handler for '/ganesh/foo' will be
# called (if they exist). 
# In the former case, the handler will be passed the "residual"
# path, thus the root handler will be passed 'ganesh/'. In the latter case,
# an empty path will be passed.

# The rationale for this is that we want to enumerate all handlers pertaining
# to the path given. Clearly anything under '/ganesh' will be relevant, but
# a higher-up handler such as the root handler might also have some useful
# information about the path and so should be given the opportunity to provide
# it.

sub forallhandlers {
  my ($type,$pathrestrict,$subtorun)=@_;
  &$error("Unknown handler type $type") unless defined $handlers{$type};

  my @components=split('/',$pathrestrict);
  shift @components; # Remove effect of leading /
  &walkhandlers_internal(1,'dynamicloader',\@components,sub {
			   return 0;
			 });

  foreach my $path (keys %{$handlers{$type}}) {
    if(index($path.'/',$pathrestrict.'/')==0) {
      foreach my $handler (@{$handlers{$type}->{$path}}) {
	&$subtorun($path,$handler->(''));
      }
    }
    elsif($pathrestrict=~/^$path\/(.*)$/) {
      foreach my $handler (@{$handlers{$type}->{$path}}) {
	&$subtorun($path,$handler->($1.'/'));
      }
    }
  }
}


1;
