package PhotoPub::Utils;
require Exporter;
@ISA = qw(Exporter);
@EXPORT_OK = qw(&slurp &doifcount &mkpath &remdups &uri_escape
		$pathre $rangere &untaint %errorstrings &checkpath &md5_uri
		&handleparameters &includetag &excludetag &handlepages);

use strict;
use PhotoPub::Global qw(&register_init $error);
use PhotoPub::FileInfo qw(&isregfile &isdir);

use Carp;
use URI::Escape 3.20 ();
use Digest::MD5 qw(&md5_base64);
use HTML::Entities;

use vars qw($pathre $rangere %errorstrings);

$pathre='[\/\-\_\w\.\#\ ]*';
$rangere='[\/\-\_\w\,\*]*';

%errorstrings=('userinput' => 'Parameter error',
	       'config' => 'Configuration error',
	       'internal' => 'Internal error',
	       'picture' => 'Problem with picture',
	       'warning' => 'Warning',
	       'unknown' => 'Error');

sub uri_escape($) {
  my $text=shift;
  # The list of safe characters is the same as used by default, with the
  # addition of '/' and ','. Although the RFC technically mandates escaping 
  # them, there seems no real reason to and it makes the links look horrible.
  return &URI::Escape::uri_escape($text,q{^A-Za-z0-9\,\-/_.!~*'()});
}

sub md5_uri($) {
  my $text=shift;
  my $md5=&md5_base64($text);
  $md5=~s{/}{-};
  $md5=~s{\+}{_};
  return $md5;
}  

sub remdups {
  my %ret=();
  foreach (@_) {
    $ret{$_}=1;
  }
  return [keys %ret];
}

sub untaint {
  my ($var,$re,$msg)=@_;
  if($$var=~/^($re)$/) {
    $$var=$1;
  } else {
    &$error('type' => 'userinput', 'message' => $msg);
  }
}

sub slurp {
  my ($filename)=@_;
  if(!open FILE,"<$filename") { 
    &carp("Couldn't open $filename");
    return undef;
  }
  my $data="";
  while(<FILE>) {
    $data.=$_;
  }
  close FILE;

  return $data;
}

sub mkpath {
  my ($basepath,$subpath)=@_;
  my @subdirs=split('/',$subpath);
  foreach my $subdir (@subdirs) {
    next if $subdir eq '';
    $basepath.="/$subdir";
    if(!-d $basepath) {
    #untaint, should really make sure this happens on input
      my ($path)=($basepath=~/^($pathre)$/)
        or &$error(type=>'internal',
                   message=>"Attempt to make invalid directory $basepath"); 
      umask 002;
      mkdir $path,04775
	or &$error(type=>'config',
		   message=>"Couldn't make directory $path");
    }
  }
}

sub doifcount {
  my ($text,$count,$total)=@_;
  my ($pre,$params,$repeat,$post);
  my $ret="";  
   
  while(($pre,$params,$repeat,$post)=($text=~/^(.*?)\<IFCOUNT(.*?)\>(.*?)\<\/IFCOUNT\>(.*)$/si)) {
    $ret.=$pre;
    if($params=~/^\s+MOD\s*(\d+)\s*\=\s*(\-?\d+)\s*$/i) {
      $ret.=$repeat if $count%$1==$2%$1;
    } elsif($params=~/^\s*\=\s*(\d+)\s*$/) {
      $ret.=$repeat if $count==$1;
    } elsif($params=~/^\s*\=\s*\-(\d+)\s*$/) {
      $ret.=$repeat if $count==$total+1-$1;
    } else { 
      &$error("couldn't parse IFCOUNT parameters");
    }
    $text=$post;
  }
  $ret.=$text;
  return $ret;
}

my %checkedcache;

&register_init('startup',sub {
		 %checkedcache=();
	       });

sub checkpath($) {
  my $file = shift;
  return 0 if !&isregfile($file) && !&isdir($file);

  my $curpath=$file;

  while(1) {
    ($curpath) = ($curpath=~ /^(.+)\//) or return 1;
    next if exists $checkedcache{$curpath};
    return 0 if $curpath =~ /\/\.\.(?:\/|$)/;
    if (-l $curpath) {
      my ($uid,$gid,$luid,$lgid);
      (undef,undef,undef,undef,$uid,$gid) = stat $curpath;
      return 0 if (!defined $uid);
      (undef,undef,undef,undef,$luid,$lgid) = lstat $curpath;
      return 0 if ($uid != $luid or $gid != $lgid);
    }
    $checkedcache{$curpath}=1;
  }
}

sub handleparameters($,\$) {
  my ($query,$templateref)=@_;
  my @params=();

  # Handle parameters declared in template file
  while($$templateref=~s/\<PARAMETER (\w+)(.*?)\>//i) {
    my $name=lc($1);
    my $extra=$2;
    my $val=$query->param($name);
    if(defined $val) {
      push @params,$name;
    } else {
      if($extra=~/DEFAULT=(.*)/i) {
	$val=$1;
      } else {
	&$error(type => 'userinput',
		message => "Must specify $name parameter for this template");
      }
    }
    $$templateref=~s/\$$name/&encode_entities($val)/ieg;
  }
  return @params;
}

sub includetag($,\$,$) {
  my ($tag,$textref,$count)=@_;
  $count=1 if !defined $count;
  $$textref=~s{<$tag>(.*?)</$tag>}{$1x$count}sieg;
}

sub excludetag($,\$) {
  &includetag(@_,0);
}

sub handlepages($,\$,\$,\$,&,&,&) {
  my ($query,$templateref,$startnumref,$endnumref,$haveprevornext,$geturl)=@_;

  my ($isprev,$isnext)=(0,0);
  my $page=1;
  my $max;

  if($$templateref=~/\<MAXITEMS (\w+)\>/i) {
    $max=$1;
    if($max!~/none/i) {
      $page=defined $query->param('page') ? $query->param('page') : 1;
      untaint(\$page,'\d+',"page must be a positive integer");
      my $startnum=($page-1)*$max;
      my $endnum=$page*$max-1;
      if($startnum<=$$startnumref) {
	$isprev=0;
      } else {
	$isprev=1;
	$$startnumref=$startnum;
      }
      if($endnum>=$$endnumref) {
	$isnext=0;
      } else {
	$isnext=1;
	$$endnumref=$endnum;
      }
    }
  }
  
  $$templateref=~s/\<MAXITEMS \w+\>//ig;
  $$templateref=~s/\$page/$page/g;

 
  if($isprev || $isnext) {
    &$haveprevornext();
    &includetag('ifpages',$templateref);
    &excludetag('nopages',$templateref);
  } else {
    &excludetag('ifpages',$templateref);
    &includetag('nopages',$templateref);
  }

  if($isnext) {
    &includetag('ifnext',$templateref);
    # $page was untainted to only contain numbers earlier
    my $url=&$geturl($page+1);
    $$templateref=~s{<a next>}{<a href=\"$url\">}sig;
  } else {
    &excludetag('ifnext',$templateref);
  }

  if($isprev) {
    &includetag('ifprev',$templateref);
    # $page was untainted to only contain numbers earlier
    my $url=&$geturl($page-1);
    $$templateref=~s{<a prev>}{<a href=\"$url\">}sig;
  } else {
    &excludetag('ifprev',$templateref);
  }
}


1;
